# Author: github.com/ductai199x
import argparse
import os
import struct

import numpy as np
import torch
from numba import njit
from tqdm.auto import tqdm


def read_header(fin):
    values = struct.unpack("i" * 9, fin.read(4 * 9))
    _, _, vocab_size, dim, multiple_of, n_heads, n_layers, rot, ftype = values
    return {
        "vocab_size": vocab_size,
        "dim": dim,
        "multiple_of": multiple_of,
        "n_heads": n_heads,
        "n_layers": n_layers,
    }, ftype


def read_tokens(fin, vocab_size):
    tokens = []
    for _ in range(vocab_size):
        text_len = struct.unpack("i", fin.read(4))[0]
        text_bytes = fin.read(text_len)
        try:
            text = text_bytes.decode()
        except UnicodeDecodeError:
            text = text_bytes.decode(errors="replace")
        score = struct.unpack("f", fin.read(4))[0]
        tokens.append((text, score))
    return tokens


@njit
def dequantize_weights_numba(fin_data, n_rows, n_cols):
    qk = 32
    nb = n_cols // qk
    bs = 4 + (qk // 2)

    weights = np.zeros((n_rows, n_cols), dtype=np.float32)
    data_pos = 0

    for row in range(n_rows):
        for block in range(nb):
            d = np.frombuffer(fin_data[data_pos : data_pos + 4], dtype=np.float32)[0]
            data_pos += 4
            packed_values = fin_data[data_pos : data_pos + (qk // 2)]
            data_pos += qk // 2

            for i in range(qk // 2):
                packed_value = packed_values[i]
                v0 = np.float32((packed_value & 0b00001111) - 8) * d
                v1 = np.float32((packed_value >> 4) - 8) * d

                weights[row, block * qk + 2 * i] = v0
                weights[row, block * qk + 2 * i + 1] = v1

    return weights


def dequantize_weights(fin, n_rows, n_cols):
    qk = 32
    nb = n_cols // qk
    data_size = n_rows * n_cols // 2 + n_rows * nb * 4
    fin_data = fin.read(data_size)
    return dequantize_weights_numba(fin_data, n_rows, n_cols)


def read_variables(fin):
    model = {}
    pbar = tqdm(total=os.path.getsize(fin.name), unit="B", unit_scale=True, desc="Reading variables")
    while True:
        start_pos = fin.tell()
        try:
            n_dims, name_length, ftype_cur = struct.unpack("iii", fin.read(4 * 3))
        except struct.error:
            break

        shape = tuple(struct.unpack("i" * n_dims, fin.read(4 * n_dims)))
        shape = shape[::-1]
        name = fin.read(name_length).decode()

        # ensure tensor data is aligned
        tensor_data_offset = fin.tell()
        tensor_data_offset = (tensor_data_offset + 31) & -32
        fin.seek(tensor_data_offset)

        if ftype_cur == 2:
            # 4-bit quantized weights
            dtype = np.uint8
            data = dequantize_weights(fin, shape[0], shape[1])
            data = data.reshape(shape)
        elif ftype_cur == 0:
            dtype = np.float32
            data_size = np.prod(shape)
            data = np.fromfile(fin, dtype=dtype, count=data_size).reshape(shape)
        elif ftype_cur == 1:
            dtype = np.float16
            data_size = np.prod(shape)
            data = np.fromfile(fin, dtype=dtype, count=data_size).reshape(shape)

        model[name] = torch.tensor(data, dtype=torch.float32 if dtype == np.float32 else torch.float16)

        pbar.update(fin.tell() - start_pos)

    return model


def convert_to_hf_format(model, hparams):
    # This works for llama 7B, need to test with other models
    n_layers = hparams["n_layers"]
    n_heads = hparams["n_heads"]
    dim = hparams["dim"]
    dims_per_head = dim // n_heads
    base = 10000.0
    inv_freq = 1.0 / (base ** (torch.arange(0, dims_per_head, 2).float() / dims_per_head))

    # permute for sliced rotary
    def permute(w):
        return w.view(n_heads, dim // n_heads // 2, 2, dim).transpose(1, 2).reshape(dim, dim)

    state_dict = {}
    for layer_i in range(n_layers):
        state_dict.update(
            {
                f"model.layers.{layer_i}.self_attn.q_proj.weight": permute(
                    model[f"layers.{layer_i}.attention.wq.weight"]
                ),
                f"model.layers.{layer_i}.self_attn.k_proj.weight": permute(
                    model[f"layers.{layer_i}.attention.wk.weight"]
                ),
                f"model.layers.{layer_i}.self_attn.v_proj.weight": model[
                    f"layers.{layer_i}.attention.wv.weight"
                ],
                f"model.layers.{layer_i}.self_attn.o_proj.weight": model[
                    f"layers.{layer_i}.attention.wo.weight"
                ],
                f"model.layers.{layer_i}.mlp.gate_proj.weight": model[
                    f"layers.{layer_i}.feed_forward.w1.weight"
                ],
                f"model.layers.{layer_i}.mlp.down_proj.weight": model[
                    f"layers.{layer_i}.feed_forward.w2.weight"
                ],
                f"model.layers.{layer_i}.mlp.up_proj.weight": model[
                    f"layers.{layer_i}.feed_forward.w3.weight"
                ],
                f"model.layers.{layer_i}.input_layernorm.weight": model[
                    f"layers.{layer_i}.attention_norm.weight"
                ],
                f"model.layers.{layer_i}.post_attention_layernorm.weight": model[
                    f"layers.{layer_i}.ffn_norm.weight"
                ],
            }
        )
        state_dict[f"model.layers.{layer_i}.self_attn.rotary_emb.inv_freq"] = inv_freq
    state_dict.update(
        {
            "model.embed_tokens.weight": model["tok_embeddings.weight"],
            "model.norm.weight": model["norm.weight"],
            "lm_head.weight": model["output.weight"],
        }
    )

    return state_dict


def chat(model, hparams, llama_dir):
    from transformers import (GenerationConfig, LlamaForCausalLM,
                              LlamaTokenizer, StoppingCriteria,
                              StoppingCriteriaList)
    from transformers.models.llama.configuration_llama import LlamaConfig

    class StoppingCriteriaSub(StoppingCriteria):
        def __init__(self):
            super().__init__()

        def __call__(self, input_ids: torch.LongTensor, scores: torch.FloatTensor, stops=[]):
            print(tokenizer.decode(input_ids[0]), end="", flush=True)
            if input_ids[0][-1] == 13:
                return True

            return False

    config = LlamaConfig(
        vocab_size=hparams["vocab_size"],
        dim=hparams["dim"],
        num_hidden_layers=hparams["n_layers"],
        num_attention_heads=hparams["n_heads"],
    )

    llama = LlamaForCausalLM(config=config)
    llama.load_state_dict(state_dict=model, strict=True)
    tokenizer = LlamaTokenizer.from_pretrained(llama_dir)

    device = torch.device("cpu")
    llama = llama.to(device)

    ctx = """You are AI.
This is a dialog, where User interacts with AI. AI is helpful, kind, obedient, honest, respectful, direct, concise, should try to protect User's privacy, and knows its own limits. Also, AI must answer User and AI cannot stop the conversation by itself.
User: Hello, AI.
AI: Hello! How can I assist you today?
"""
    print(ctx.rstrip("\n"))
    while True:
        print("-" * 60)
        prompt = input("User: ")
        if ctx != "":
            ctx = f"{ctx}User: {prompt}\n"
        else:
            ctx = f"{prompt}\nAI:"

        ctx = (ctx[-1920:]) if len(ctx) >= 2048 else ctx

        print("-" * 60)
        if len(ctx.strip()) > 0:
            input_ids = tokenizer(ctx, return_tensors="pt")["input_ids"].to(device)
            generation_config = GenerationConfig(
                temperature=0.8,
                top_p=0.95,
                top_k=50,
                repetition_penalty=1.1764,
            )
            with torch.no_grad():
                generation_output = llama.generate(
                    input_ids=input_ids,
                    generation_config=generation_config,
                    return_dict_in_generate=True,
                    output_scores=True,
                    max_length=2048,
                    do_sample=True,
                    stopping_criteria=StoppingCriteriaList([StoppingCriteriaSub()]),
                )
            s = generation_output.sequences[0]
            decoded = tokenizer.decode(s)
            ctx = f"{decoded}\n"


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--input_dir", "-i", type=str, required=True, help="The input directory containing the ggml files."
    )
    parser.add_argument(
        "--prefix",
        "-p",
        type=str,
        required=True,
        help="The prefix of the ggml files (ggml-model-f16 or ggml-model-q4_0).",
    )
    parser.add_argument(
        "--hf",
        action="store_true",
        help="Whether to save the model in the Hugging Face format. (default: False)",
    )
    parser.add_argument(
        "--chat", "-c", action="store_true", help="Whether to open a chat with the model. (default: False)"
    )
    args = parser.parse_args()

    llama_dir = os.path.abspath(f"{args.input_dir}/../")

    ggml_files = sorted(
        [f"{args.input_dir}/{f}" for f in os.listdir(args.input_dir) if f.startswith(args.prefix)]
    )

    fin = open(ggml_files[0], "rb")
    hparams, ftype = read_header(fin)
    tokens = read_tokens(fin, hparams["vocab_size"])
    model = read_variables(fin)

    for f in tqdm(ggml_files[1:]):
        fin = open(f, "rb")
        read_header(fin)
        read_tokens(fin, hparams["vocab_size"])
        model.update(read_variables(fin))

    if args.hf:
        model = convert_to_hf_format(model, hparams)

    pth_ckpt = {
        "state_dict": model,
        "hparams": hparams,
        "tokens": tokens,
    }

    torch.save(pth_ckpt, f"{args.input_dir}/{args.prefix}-to-torch.pth")

    if args.chat:
        if not args.hf:
            model = convert_to_hf_format(model, hparams)
        chat(model, hparams, llama_dir)


if __name__ == "__main__":
    main()



import os
import sys
import array
import llamacpp
import pytest


@pytest.fixture(scope="session")
def llama_context():
    params = llamacpp.LlamaContextParams()
    params.seed = 19472
    # Get path to current file
    current_file_path = os.path.dirname(os.path.realpath(__file__))
    # Get path to the model
    model_path = os.path.join(current_file_path, "../models/7B/ggml-model-f16.bin")
    return llamacpp.LlamaContext(model_path, params)


def test_str_to_token(llama_context):
    prompt = "Hello World"
    prompt_tokens = llama_context.str_to_token(prompt, True)
    assert all(prompt_tokens == [1, 10994, 2787])


def test_token_to_str(llama_context):
    tokens = [1, 10994, 2787]
    text = ''.join([llama_context.token_to_str(token) for token in tokens])
    assert text == "Hello World"


def test_eval(llama_context):
    embd_inp = llama_context.str_to_token(" Llama is", True)
    n_past, n_remain, n_consumed  = 0, 8, 0
    embd = []

    output = ''
    while n_remain:
        if len(embd):
            llama_context.eval(array.array('i', embd), len(embd), n_past, 1)
        n_past += len(embd)
        embd.clear()

        if len(embd_inp) <= n_consumed:
            # sample
            top_k = 40
            top_p = 0.95
            temp = 0.8
            repeat_penalty = 1.0

            # sending an empty array for the last n tokens
            id = llama_context.sample_top_p_top_k(array.array('i', []), top_k, top_p, temp, repeat_penalty)
            # add it to the context
            embd.append(id)
            # decrement remaining sampling budget
            n_remain -= 1
        else:
            # has unconsumed input
            while len(embd_inp) > n_consumed:
                # update_input
                embd.append(embd_inp[n_consumed])
                n_consumed += 1

        output += ''.join([llama_context.token_to_str(id) for id in embd])
    assert output == " Llama is the newest member of our growing family"


if __name__=='__main__':
    sys.exit(pytest.main(['-s', '-v', __file__]))



import os
import sys


def main():
    """Pass command line arguments to llama_model_quantize"""
    import llamacpp

    # Print usage if not enough arguments are provided
    if len(sys.argv) < 2:
        print(f"Usage: llamacpp-quantize <model_path> [<bits>=0]")
        print("bits: 0 = q4_0, 1 = q4_1\n")
        print("This script assumes that you have already used convert-pth-to-ggml.py to convert")
        print("the pytorch model to a ggml model. It will then quantize the ggml model to INT4")
        print("for use with the llamacpp library.\n")
        print("llamacpp-quantize will walk through the model_path directory and quantize all")
        print("ggml-model-f16.bin.* files it finds. The output files will be named")
        print("ggml-model-q4_0.bin.* or ggml-model-q4_1.bin.* depending on the value of <bits>.\n")
        sys.exit(1)
    
    model_path = sys.argv[1]
    if len(sys.argv) < 3:
        bits = 0
    else:
        bits = int(sys.argv[2])

    # Convert "bits" to input for llama_model_quantize()
    if bits == 0:
        q_type = 2
        q_type_str = 'q4_0'
    elif bits == 1:
        q_type = 3
        q_type_str = 'q4_1'

    # Print the model path
    print(f"Quantizing model in {model_path} to {q_type_str}")

    # Walk through files in model_path matching ggml-model-q*.bin 
    # and pass them to llama_model_quantize()
    for root, dirs, files in os.walk(model_path):
        for file in files:
            if file.startswith("ggml-model-f16") and file.endswith(".bin"):               
                output_file = file.replace("-f16.bin", f"-{q_type_str}.bin")
                print(f"Quantizing file: {file} to {output_file}")
                llamacpp.llama_model_quantize(os.path.join(root, file), os.path.join(root, output_file), q_type)


if __name__ == '__main__':
    main()



"""A modified version of llamacpp-cli and includes a good prompt for the chatbot"""
import sys
import llamacpp
import argparse
from typing import Dict

from llamacpp.cli import main as llamacpp_main

# Default prompt
prompt = """Transcript of a dialog, where the User interacts with an Assistant named Bob. Bob is helpful, kind, honest, good at writing, and never fails to answer the User's requests immediately and with precision.

User: Hello, Bob.
Bob: Hello. How may I help you today?
User:"""


def parse_chat_params(argv) -> Dict[str, str]:
    """Parse chat parameters"""

    parser = argparse.ArgumentParser(description="LLaMa")
    parser.add_argument("-i", "--interactive", action="store_true", help="run in interactive mode", default=True)
    parser.add_argument(
        "--interactive-start",
        action="store_true",
        help="run in interactive mode and poll user input at startup",
        default=False,
    )
    parser.add_argument(
        "-r",
        "--reverse-prompt",
        type=str,
        help="in interactive mode, poll user input upon seeing PROMPT",
        default="User:",
    )
    parser.add_argument(
        "--color",
        action="store_true",
        help="colorise output to distinguish prompt and user input from generations",
        default=True,
    )
    parser.add_argument("-s", "--seed", type=int, default=-1, help="RNG seed (default: -1)")
    parser.add_argument(
        "-t",
        "--threads",
        type=int,
        default=8,
        help="number of threads to use during computation (default: 4)",
    )
    parser.add_argument(
        "-p",
        "--prompt",
        type=str,
        help="prompt to start generation with (default: random)",
        default=prompt,
    )
    # parser.add_argument(
    #     "-f", "--file", type=str, default="", help="prompt file to start generation."
    # )
    parser.add_argument(
        "-n", "--n_predict", type=int, default=256, help="number of tokens to predict (default: 128)"
    )
    parser.add_argument("--top_k", type=int, default=40, help="top-k sampling (default: 40)")
    parser.add_argument("--top_p", type=float, default=0.95, help="top-p sampling (default: 0.1)")
    parser.add_argument(
        "--repeat_last_n",
        type=int,
        default=64,
        help="last n tokens to consider for penalize (default: 0)",
    )
    parser.add_argument(
        "--repeat_penalty",
        type=float,
        default=1.30,
        help="penalize repeat sequence of tokens (default: 0.0)",
    )
    parser.add_argument(
        "-c",
        "--ctx_size",
        type=int,
        default=4096,
        help="size of the prompt context (default: 4096)",
    )
    parser.add_argument("--temp", type=float, default=0.8, help="temperature (default: 0.7)")
    parser.add_argument(
        "-b",
        "--batch_size",
        type=int,
        default=8,
        help="batch size for prompt processing (default: 2)",
    )
    parser.add_argument("-m", "--model", type=str, default="./models/7B/ggml-model-q4_0.bin", help="model path (default: )")
    parser.add_argument("--mlock", action="store_true", help="use mlock to lock memory")
    parser.add_argument("--memory_f16", action="store_true", help="use half-precision memory")

    args = parser.parse_args(argv[1:])

    return args


def run():
    args = parse_chat_params(sys.argv)

    args.instruct = False

    return llamacpp_main(args)


if __name__ == "__main__":
    sys.exit(run())



"""Python version of main.cpp"""
import sys
import argparse
import llamacpp
from typing import Dict


def parse_args_into_params(argv) -> Dict[str, str]:
    """Parses arguments using argparse based on usage information above"""
    parser = argparse.ArgumentParser(description="llama.cpp CLI")
    parser.add_argument("-i", "--interactive", action="store_true", help="run in interactive mode")
    parser.add_argument(
        "-ins", "--instruct",
        action="store_true",
        help="run in 'instruct mode' where the user is prompted to enter a command",
        default=False,
    )
    parser.add_argument(
        "-r",
        "--reverse-prompt",
        type=str,
        help="in interactive mode, poll user input upon seeing PROMPT",
        default="",
    )
    parser.add_argument(
        "--color",
        action="store_true",
        help="colorise output to distinguish prompt and user input from generations",
    )
    parser.add_argument("-s", "--seed", type=int, default=-1, help="RNG seed (default: -1)")
    parser.add_argument(
        "-t",
        "--threads",
        type=int,
        default=4,
        help="number of threads to use during computation (default: 4)",
    )
    parser.add_argument(
        "-p",
        "--prompt",
        type=str,
        help="prompt to start generation with (default: random)",
    )
    parser.add_argument(
        "-f", "--file", type=str, default="", help="prompt file to start generation."
    )
    parser.add_argument(
        "-n", "--n_predict", type=int, default=128, help="number of tokens to predict (default: 128)"
    )
    parser.add_argument("--top_k", type=int, default=40, help="top-k sampling (default: 40)")
    parser.add_argument("--top_p", type=float, default=0.95, help="top-p sampling (default: 0.1)")
    parser.add_argument(
        "--repeat_last_n",
        type=int,
        default=64,
        help="last n tokens to consider for penalize (default: 64)",
    )
    parser.add_argument(
        "--repeat_penalty",
        type=float,
        default=1.30,
        help="penalize repeat sequence of tokens (default: 1.30)",
    )
    parser.add_argument(
        "-c",
        "--ctx_size",
        type=int,
        default=512,
        help="size of the prompt context (default: 512)",
    )
    parser.add_argument("--temp", type=float, default=0.8, help="temperature (default: 0.7)")
    parser.add_argument(
        "-b",
        "--batch_size",
        type=int,
        default=8,
        help="batch size for prompt processing (default: 8)",
    )
    parser.add_argument("-m", "--model", type=str, default="./models/7B/ggml-model-q4_0.bin", help="model path (default: )")
    parser.add_argument("--mlock", action="store_true", help="use mlock to lock memory")
    parser.add_argument("--memory_f16", action="store_true", help="use half-precision memory")

    args = parser.parse_args(argv[1:])

    if args.interactive or args.instruct:
        print("WARNING: interactive mode and instruct mode are currently broken")
    return args


def process_interactive_input(model: llamacpp.LlamaInference):
    """Process interactive input similar to the C++ version"""

    # Read lines as long as user is entering "\" at the end of the line
    # Pass each line to the model
    while True:
        line = input()
        if line.endswith("\\"):
            line = line[:-1]
            model.update_input(line)
        else:
            model.update_input(line)
            break


def main(args):
    """Main function"""

    # Add a space in front of the first character to match OG llama tokenizer behavior
    args.prompt = " " + args.prompt
    
    params = llamacpp.InferenceParams()
    params.path_model = args.model
    params.seed = args.seed
    params.n_threads = args.threads

    params.repeat_last_n = args.repeat_last_n
    params.n_batch = args.batch_size
    params.top_k = args.top_k
    params.top_p = args.top_p
    params.temp = args.temp
    params.repeat_penalty = args.repeat_penalty
    params.use_mlock = args.mlock
    params.memory_f16 = args.memory_f16
    params.n_ctx = args.ctx_size

    model = llamacpp.LlamaInference(params)
    model.update_input([model.token_bos()])
    model.update_input(args.prompt)
    print(model.system_info())

    inp_pfx = model.tokenize("\n\n### Instruction:\n\n", True)
    inp_sfx = model.tokenize("\n\n### Response:\n\n", False)

    if args.instruct:
        args.interactive = True
        args.reverse_prompt = "### Instruction:\n\n"

    # Set antiprompt if we are in interactive mode
    if args.reverse_prompt:
        args.interactive = True

    if args.interactive:
        print("== Running in interactive mode. ==")
        print(" - Press Ctrl+C to interject at any time.")
        print(" - Press Return to return control to LLaMa.")
        print(" - If you want to submit another line, end your input in '\\'.")
        print()
        is_interacting = True

    input_noecho = False
    is_finished = False

    print(args.prompt, end="")

    n_output = 0
    while n_output < args.n_predict:
        if model.has_unconsumed_input():
            model.ingest_all_pending_input()
            # # reset color to default if we there is no pending user input
            # if (!input_noecho && args.use_color) {
            #     printf(ANSI_COLOR_RESET);
            # }
        else:
            token = model.sample()
            text = model.token_to_str(token)
            print(text, end="")
            n_output += 1
            is_finished = token == model.token_eos()
            input_noecho = False

        if args.interactive:
            if model.is_antiprompt_present():
                # reverse prompt found
                is_interacting = True
            if is_interacting:
                if args.instruct:
                    model.update_input_tokens(inp_pfx)
                    print("\n> ", end="")

                process_interactive_input(model)

                if args.instruct:
                    model.update_input_tokens(inp_sfx)

                input_noecho = True
                is_interacting = False
        
        # end of text token was found
        if is_finished:
            if args.interactive:
                is_interacting = True
            else:
                print(" [end of text]")
                break
        
        if args.interactive and model.is_finished():
            model.reset_remaining_tokens()
            is_interacting = True

    return 0


def run():
    # Parse params into a gpt_params object
    args = parse_args_into_params(sys.argv)

    # if args.file is specified, read the file and set the prompt to the contents
    if args.file:
        with open(args.file, "r") as f:
            args.prompt = f.read().strip()

    return main(args)


if __name__ == "__main__":
    sys.exit(run())



import sys
import llamacpp


def progress_callback(progress):
    # print("Progress: {:.2f}%".format(progress * 100))
    # sys.stdout.flush()
    pass


params = llamacpp.InferenceParams.default_with_callback(progress_callback)
params.path_model = './models/7B/ggml-model-f16.bin'
params.seed = 19472
params.repeat_penalty = 1.0
model = llamacpp.LlamaInference(params)

prompt = " Llama is"
prompt_tokens = model.tokenize(prompt, True)
model.update_input(prompt_tokens)

model.ingest_all_pending_input()
print(model.system_info())

print(prompt, end='')
for i in range(20):
    model.eval()
    token = model.sample()
    text = model.token_to_str(token)
    print(text, end="")
    
# Flush stdout
sys.stdout.flush()

# model.print_timings()



import array
import llamacpp

params = llamacpp.LlamaContextParams()
params.seed = 19472
model = llamacpp.LlamaContext("./models/7B/ggml-model-f16.bin", params)

prompt = "Llama is"
# add a space in front of the first character to match OG llama tokenizer behavior
prompt = f" {prompt}"

# tokenize the prompt
embd_inp = model.str_to_token(prompt, True)

n_ctx = model.get_n_ctx()

if len(embd_inp) > n_ctx - 4:
    raise Exception("Prompt is too long")

n_past = 0
n_remain = 9
n_consumed = 0
embd = []

while n_remain:
    if len(embd):
        if model.eval(array.array('i', embd), len(embd), n_past, 1):
            raise Exception("Failed to predict\n")
    n_past += len(embd)
    embd.clear()

    if len(embd_inp) <= n_consumed:
        # sample
        top_k = 40
        top_p = 0.95
        temp = 0.8
        repeat_penalty = 0.0

        # sending an empty array for the last n tokens
        id = model.sample_top_p_top_k(array.array('i', []), top_k, top_p, temp, repeat_penalty)
        # add it to the context
        embd.append(id)
        # decrement remaining sampling budget
        n_remain -= 1
    else:
        # has unconsumed input
        while len(embd_inp) > n_consumed:
            # update_input
            embd.append(embd_inp[n_consumed])
            n_consumed += 1

    for id in embd:
        print(model.token_to_str(id), end="")



"""Demonstrates that the library now supports going over the context size limit 
(but loses "memory" of earlier text in the process)"""
import sys
import llamacpp


params = llamacpp.InferenceParams()
params.path_model = './models/7B/ggml-model-f16.bin'
params.seed = 69420
params.repeat_penalty = 1.0
params.n_ctx = 128
model = llamacpp.LlamaInference(params)

prompt = " Llama is"
prompt_tokens = model.tokenize(prompt, True)
model.update_input(prompt_tokens)

model.ingest_all_pending_input()
print(model.system_info())

print(prompt, end='')
for i in range(256):
    model.eval()
    token = model.sample()
    text = model.token_to_str(token)
    print(text, end='')
    sys.stdout.flush()

print()
model.print_timings()



import os
import sys
import pytest
import llamacpp


@pytest.fixture
def nominal_params():
    """Nominal InfereceParams"""
    params = llamacpp.InferenceParams()
    # Get path to current file
    current_file_path = os.path.dirname(os.path.realpath(__file__))
    # Get path to the model
    model_path = os.path.join(current_file_path, "../models/7B/ggml-model-f16.bin")
    params.path_model = model_path
    params.seed = 19472
    params.repeat_penalty = 1.0
    return params


@pytest.fixture
def limited_context_params(nominal_params):
    """InferenceParams with limited context"""
    nominal_params.n_ctx = 32
    return nominal_params


@pytest.fixture
def llama_model(nominal_params):
    return llamacpp.LlamaInference(nominal_params)


@pytest.fixture
def limited_context_llama_model(limited_context_params):
    return llamacpp.LlamaInference(limited_context_params)


def test_update_input(llama_model):
    prompt_tokens = [1, 2, 3]
    llama_model.update_input(prompt_tokens)
    assert llama_model.has_unconsumed_input()
    llama_model.ingest_all_pending_input()
    assert not llama_model.has_unconsumed_input()


def test_tokenize(llama_model):
    prompt = "Hello World"
    prompt_tokens = llama_model.tokenize(prompt, True)
    assert prompt_tokens == [1, 10994, 2787]


def test_token_to_str(llama_model):
    tokens = [1, 10994, 2787]
    text = ''.join([llama_model.token_to_str(token) for token in tokens])
    assert text == "Hello World"


def test_eval(llama_model):
    prompt = " Llama is"
    prompt_tokens = llama_model.tokenize(prompt, True)
    llama_model.update_input(prompt_tokens)
    llama_model.ingest_all_pending_input()
    output = prompt
    for i in range(8):
        llama_model.eval()
        token = llama_model.sample()
        output += llama_model.token_to_str(token)

    assert output == " Llama is the newest member of our growing family"


def test_eval_exceed_n_ctx(limited_context_llama_model):
    # Tests context swapping feature
    llama_model = limited_context_llama_model
    prompt = " Llama is"
    prompt_tokens = llama_model.tokenize(prompt, True)
    llama_model.update_input(prompt_tokens)
    llama_model.ingest_all_pending_input()
    output = prompt
    # Generate 35 tokens with n_ctx of 32
    for i in range(35):
        llama_model.eval()
        token = llama_model.sample()
        output += llama_model.token_to_str(token)

    assert output == " Llama is the newest member of our growing family. We’re excited to welcome him to the pack!\nLlama is a male, born in 2017"


if __name__=='__main__':
    sys.exit(pytest.main(['-s', '-v', __file__]))
